@@ -3,6 +3,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE UndecidableInstances       #-}
 -----------------------------------------------------------------------------
@@ -25,7 +26,6 @@ module Diagrams.Core.Envelope
        ( -- * Envelopes
          Envelope(..)
 
-       , inEnvelope
        , appEnvelope
        , onEnvelope
        , mkEnvelope
@@ -36,6 +36,7 @@ module Diagrams.Core.Envelope
          -- * Utility functions
        , diameter
        , radius
+       , extent
        , envelopeVMay, envelopeV, envelopePMay, envelopeP, envelopeSMay, envelopeS
 
          -- * Miscellaneous
@@ -43,12 +44,13 @@ module Diagrams.Core.Envelope
        ) where
 
 import           Control.Applicative     ((<$>))
+import           Control.Lens            (Rewrapped, Wrapped (..), iso, mapped,
+                                          op, over, _Wrapping')
 import qualified Data.Map                as M
 import           Data.Maybe              (fromMaybe)
 import           Data.Semigroup
 import qualified Data.Set                as S
 
-import           Data.AffineSpace        ((.+^), (.-^))
 import           Data.VectorSpace
 
 import           Diagrams.Core.HasOrigin
@@ -94,17 +96,19 @@ import           Diagrams.Core.V
 --   The idea for envelopes came from
 --   Sebastian Setzer; see
 --   <http://byorgey.wordpress.com/2009/10/28/collecting-attributes/#comment-2030>.  See also Brent Yorgey, /Monoids: Theme and Variations/, published in the 2012 Haskell Symposium: <http://www.cis.upenn.edu/~byorgey/pub/monoid-pearl.pdf>; video: <http://www.youtube.com/watch?v=X-8NCkD2vOw>.
-newtype Envelope v = Envelope { unEnvelope :: Option (v -> Max (Scalar v)) }
+newtype Envelope v = Envelope (Option (v -> Max (Scalar v)))
 
-inEnvelope :: (Option (v -> Max (Scalar v)) -> Option (v -> Max (Scalar v)))
-           -> Envelope v -> Envelope v
-inEnvelope f = Envelope . f . unEnvelope
+instance Wrapped (Envelope v) where
+    type Unwrapped (Envelope v) = Option (v -> Max (Scalar v))
+    _Wrapped' = iso (\(Envelope e) -> e) Envelope
+
+instance Rewrapped (Envelope v) (Envelope v')
 
 appEnvelope :: Envelope v -> Maybe (v -> Scalar v)
 appEnvelope (Envelope (Option e)) = (getMax .) <$> e
 
 onEnvelope :: ((v -> Scalar v) -> (v -> Scalar v)) -> Envelope v -> Envelope v
-onEnvelope t = (inEnvelope . fmap) ((Max .) . t . (getMax .))
+onEnvelope t = over (_Wrapping' Envelope . mapped) ((Max .) . t . (getMax .))
 
 mkEnvelope :: (v -> Scalar v) -> Envelope v
 mkEnvelope = Envelope . Option . Just . (Max .)
@@ -185,7 +189,7 @@ instance (OrderedField (Scalar v), InnerSpace v) => Enveloped (Point v) where
   getEnvelope p = moveTo p . mkEnvelope $ const zeroV
 
 instance Enveloped t => Enveloped (TransInv t) where
-  getEnvelope = getEnvelope . unTransInv
+  getEnvelope = getEnvelope . op TransInv
 
 instance (Enveloped a, Enveloped b, V a ~ V b) => Enveloped (a,b) where
   getEnvelope (x,y) = getEnvelope x <> getEnvelope y
@@ -252,11 +256,16 @@ envelopeS v = fromMaybe 0 . envelopeSMay v
 -- | Compute the diameter of a enveloped object along a particular
 --   vector.  Returns zero for the empty envelope.
 diameter :: Enveloped a => V a -> a -> Scalar (V a)
-diameter v a = case appEnvelope $ getEnvelope a of
-  (Just env) -> (env v + env (negateV v)) * magnitude v
-  Nothing -> 0
+diameter v a = maybe 0 (\(lo,hi) -> (hi - lo) * magnitude v) (extent v a)
 
 -- | Compute the \"radius\" (1\/2 the diameter) of an enveloped object
 --   along a particular vector.
 radius :: Enveloped a => V a -> a -> Scalar (V a)
-radius v = (0.5*) . diameter v
\ No newline at end of file
+radius v = (0.5*) . diameter v
+
+-- | Compute the range of an enveloped object along a certain
+--   direction.  Returns a pair of scalars @(lo,hi)@ such that the
+--   object extends from @(lo *^ v)@ to @(hi *^ v)@. Returns @Nothing@
+--   for objects with an empty envelope.
+extent :: Enveloped a => V a -> a -> Maybe (Scalar (V a), Scalar (V a))
+extent v a = (\f -> (-f (negateV v), f v)) <$> (appEnvelope . getEnvelope $ a)
