@@ -5,6 +5,8 @@
            , TypeFamilies
            , MultiParamTypeClasses
            , GeneralizedNewtypeDeriving
+           , TemplateHaskell
+           , TypeFamilies
            , TypeSynonymInstances
            , ScopedTypeVariables
   #-}
@@ -35,10 +37,18 @@ module Diagrams.Core.Transform
          -- ** General transformations
        , Transformation(..)
        , inv, transp, transl
+       , dropTransl
        , apply
        , papply
        , fromLinear
+       , basis
+       , dimension
        , onBasis
+       , listRep
+       , matrixRep
+       , matrixHomRep
+       , determinant
+       , avgScale
 
          -- * The Transformable class
 
@@ -47,7 +57,7 @@ module Diagrams.Core.Transform
 
          -- * Translational invariance
 
-       , TransInv(..)
+       , TransInv(TransInv)
 
          -- * Vector space independent transformations
          -- | Most transformations are specific to a particular vector
@@ -59,6 +69,7 @@ module Diagrams.Core.Transform
 
        ) where
 
+import           Control.Lens                 (Wrapped(..), Rewrapped, iso)
 import qualified Data.Map as M
 import           Data.Semigroup
 import qualified Data.Set as S
@@ -131,6 +142,9 @@ lapp (f :-: _) = lapply f
 --   inverse transpose.  This is exactly what we need when
 --   transforming bounding functions, which are defined in terms of
 --   /perpendicular/ (i.e. normal) hyperplanes.
+--
+--   For more general, non-invertable transformations, see
+--   @Diagrams.Deform@ (in @diagrams-lib@).
 
 data Transformation v = Transformation (v :-: v) (v :-: v) v
 
@@ -150,6 +164,11 @@ transp (Transformation _ t' _) = t'
 transl :: Transformation v -> v
 transl (Transformation _ _ v) = v
 
+-- | Drop the translational component of a transformation, leaving only
+--   the linear part.
+dropTransl :: AdditiveGroup v => Transformation v -> Transformation v
+dropTransl (Transformation a a' _) = Transformation a a' zeroV
+
 -- | Transformations are closed under composition; @t1 <> t2@ is the
 --   transformation which performs first @t2@, then @t1@.
 instance HasLinearMap v => Semigroup (Transformation v) where
@@ -181,19 +200,95 @@ papply (Transformation t _ v) (P p) = P $ lapp t p ^+^ v
 fromLinear :: AdditiveGroup v => (v :-: v) -> (v :-: v) -> Transformation v
 fromLinear l1 l2 = Transformation l1 l2 zeroV
 
+-- | Get the matrix equivalent of the basis of the vector space v as
+--   a list of columns.
+basis :: forall v. HasLinearMap v => [v]
+basis = map basisValue b
+  where b = map fst (decompose (zeroV :: v))
+
+-- | Get the dimension of an object whose vector space is an instance of
+--   @HasLinearMap@, e.g. transformations, paths, diagrams, etc.
+dimension :: forall a. HasLinearMap (V a) => a -> Int
+dimension _ = length (decompose (zeroV :: V a))
+
 -- | Get the matrix equivalent of the linear transform,
 --   (as a list of columns) and the translation vector.  This
 --   is mostly useful for implementing backends.
 onBasis :: forall v. HasLinearMap v => Transformation v -> ([v], v)
 onBasis t = (vmat, tr)
-  where tr :: v
-        tr    = transl t
-        basis :: [Basis v]
-        basis = map fst (decompose tr)
-        es :: [v]
-        es    = map basisValue basis
-        vmat :: [v]
-        vmat = map (apply t) es
+  where
+      tr    = transl t
+      vmat = map (apply t) basis
+
+-- Remove the nth element from a list
+remove :: Int -> [a] -> [a]
+remove n xs = ys ++ (tail zs)
+  where
+    (ys, zs) = splitAt n xs
+
+-- Minor matrix of cofactore C(i,j)
+minor :: Int -> Int -> [[a]] -> [[a]]
+minor i j xs = remove j $ map (remove i) xs
+
+-- The determinant of a square matrix represented as a list of lists
+-- representing column vectors, that is [column].
+det :: Num a => [[a]] -> a
+det (a:[]) = head a
+det m = sum [(-1)^i * (c1 !! i) * det (minor i 0 m) | i <- [0 .. (n-1)]]
+  where
+    c1 = head m
+    n = length m
+
+-- | Convert a vector v to a list of scalars.
+listRep :: HasLinearMap v => v -> [Scalar v]
+listRep v = map snd (decompose v)
+
+-- | Convert a `Transformation v` to a matrix representation as a list of
+--   column vectors which are also lists.
+matrixRep :: HasLinearMap v => Transformation v -> [[Scalar v]]
+matrixRep t = map listRep (fst . onBasis $ t)
+
+-- | Convert a `Transformation v` to a homogeneous matrix representation.
+--   The final list is the translation.
+--   The representation leaves off the last row of the matrix as it is
+--   always [0,0, ... 1] and this representation is the defacto standard
+--   for backends.
+matrixHomRep :: HasLinearMap v => Transformation v -> [[Scalar v]]
+matrixHomRep t = mr ++ [listRep tl]
+  where
+    mr = matrixRep t
+    tl = transl t
+
+-- | The determinant of a `Transformation`.
+determinant :: (HasLinearMap v, Num (Scalar v)) => Transformation v -> Scalar v
+determinant t = det . matrixRep $ t
+
+-- | Compute the \"average\" amount of scaling performed by a
+--   transformation.  Satisfies the properties
+--
+--   @
+--   avgScale (scaling k) == k
+--   avgScale (t1 <> t2)  == avgScale t1 * avgScale t2
+--   @
+--
+avgScale :: (HasLinearMap v, Floating (Scalar v)) => Transformation v -> Scalar v
+avgScale t = (abs . determinant $ t) ** (1 / fromIntegral (dimension t))
+
+{-
+
+avgScale is computed as the nth root of the positive determinant.
+This works because the determinant is the factor by which a transformation
+scales area/volume. See http://en.wikipedia.org/wiki/Determinant.
+
+Proofs for the specified properties:
+
+1. |det (scaling k)|^(1/n) = (k^n)^(1/n) = k
+2. |det t1|^(1/n) * |det t2|^(1/n)
+   = (|det t1| * |det t2|)^(1/n)
+   = |det t1 * det t2|^(1/n)
+   = |det (t1 <> t2)|^(1/n)
+
+-}
 
 ------------------------------------------------------------
 --  The Transformable class  -------------------------------
@@ -270,9 +365,15 @@ instance Transformable Rational where
 --   translationally invariant; the translational component of
 --   transformations will no longer affect things wrapped in
 --   @TransInv@.
-newtype TransInv t = TransInv { unTransInv :: t }
+newtype TransInv t = TransInv t
   deriving (Eq, Ord, Show, Semigroup, Monoid)
 
+instance Wrapped (TransInv t) where
+    type Unwrapped (TransInv t) = t
+    _Wrapped' = iso (\(TransInv t) -> t) TransInv
+
+instance Rewrapped (TransInv t) (TransInv t')
+
 type instance V (TransInv t) = V t
 
 instance VectorSpace (V t) => HasOrigin (TransInv t) where
